@@ -2,7 +2,7 @@
 #   nncross.R
 #
 #
-#    $Revision: 1.39 $  $Date: 2022/05/21 09:52:11 $
+#    $Revision: 1.43 $  $Date: 2024/02/02 02:37:06 $
 #
 #  Copyright (C) Adrian Baddeley, Jens Oehlschlaegel and Rolf Turner 2000-2012
 #  Licence: GNU Public Licence >= 2
@@ -135,10 +135,11 @@ nncross.ppp <- function(X, Y, iX=NULL, iY=NULL,
     if(exclude) iY <- iY[oY]
   }
 
-  #' upper bound on distance
-  dmax <- diameter(boundingbox(as.rectangle(X), as.rectangle(Y)))
-  huge <- 1.1 * dmax
-
+  #' Largest possible distance computable in double precision
+  huge <- sqrt(.Machine$double.xmax)
+  #' Code initialises nndist^2 to huge^2
+  #' and returns nndist='huge' whenever the set of distances is empty
+  huge <- 0.999 * huge # Ensures huge^2 <= .Machine$double.xmax
   
   # number of neighbours that are well-defined
   kmaxcalc <- min(nY, kmax)
@@ -176,7 +177,7 @@ nncross.ppp <- function(X, Y, iX=NULL, iY=NULL,
                 call.=FALSE)
       nnwcode[uhoh] <- NA
       if(want.dist) nndval[uhoh] <- Inf
-    } else if(want.dist && any(uhoh <- (nndval > dmax))) {
+    } else if(want.dist && any(uhoh <- (nndval > 0.99 * huge))) {
       if(!exclude)
         warning("Infinite distances unexpectedly returned in nncross",
                 call.=FALSE)
@@ -227,7 +228,7 @@ nncross.ppp <- function(X, Y, iX=NULL, iY=NULL,
     if(want.which && any(uhoh <- (nnW == 0))) {
       nnW[uhoh] <- NA
       if(want.dist) nnD[uhoh] <- Inf
-    } else if(want.dist && any(uhoh <- (nnD > dmax)))
+    } else if(want.dist && any(uhoh <- (nnD > 0.99 * huge)))
       nnD[uhoh] <- Inf
     
     # reinterpret indices in original ordering
