@@ -4,21 +4,22 @@
 ##  'persp' method for image objects
 ##      plus annotation
 ##  
-##  $Revision: 1.28 $ $Date: 2022/05/23 02:33:06 $
+##  $Revision: 1.31 $ $Date: 2025/09/09 23:11:52 $
 ##
 
 persp.im <- function(x, ...,
                      colmap=NULL, colin=x, apron=FALSE,
                      visible=FALSE) {
-  xname <- deparse(substitute(x))
+  xname <- short.deparse(substitute(x))
   xinfo <- summary(x)
   if(xinfo$type == "factor")
     stop("Perspective plot is inappropriate for factor-valued image")
+  pop <- spatstat.options("par.persp")
   ## check whether 'col' was specified when 'colmap' was intended
   Col <- list(...)$col
   if(is.null(colmap) && !is.null(Col) && !is.matrix(Col) && length(Col) != 1)
     warning("Argument col is not a matrix. Did you mean colmap?")
-  if(!missing(colin)) {
+  if(has.colin <- !missing(colin)) {
     ## separate image to determine colours
     verifyclass(colin, "im")
     if(!compatible(colin, x)) {
@@ -28,7 +29,36 @@ persp.im <- function(x, ...,
     if(is.null(colmap))
       colmap <- spatstat.options("image.colfun")(128)
   }
-  pop <- spatstat.options("par.persp")
+  ## modify image data to add an apron
+  if(apron) {
+    zlim <- list(...)$zlim
+    bottom <- if(!is.null(zlim)) zlim[1] else min(x)
+    if(has.colin)
+      mincolin <- min(colin, na.rm=TRUE, finite=TRUE)
+    xmask   <- as.mask(x)
+    isapron <- as.im(FALSE, W=xmask)
+    if(!anyNA(x)) {
+      ## add 1 pixel at border
+      x       <- padimage(x, bottom)
+      isapron <- padimage(isapron, TRUE)
+      if(has.colin)
+        colin <- padimage(colin, mincolin)
+    } else {
+      ## add about 4 pixels around domain of x
+      r <- with(x, 4 * sqrt(xstep^2+ystep^2))
+      V <- dilation(xmask, r)
+      x       <- padimage(x, bottom, W=V)
+      isapron <- padimage(isapron, TRUE, W=V)
+      if(has.colin)
+        colin <- padimage(colin, mincolin, W=V)
+      xold   <- x[xmask, drop=FALSE]
+      xclose <- nearestValue(xold)
+      transition <- 1 - distmap(as.owin(xold))/r
+      transition <- eval.im(pmax(0, pmin(1, transition)))
+      x[isapron] <- (xclose * transition + bottom * (1-transition))[isapron]
+    }
+    xinfo <- summary(x)
+  }
   ##
   if(is.function(colmap) && !inherits(colmap, "colourmap")) {
     ## coerce to a 'colourmap' if possible
@@ -54,13 +84,13 @@ persp.im <- function(x, ...,
     colval <- t(as.matrix(colval))
     ## strip one row and column for input to persp.default
     colval <- colval[-1, -1]
-    ## replace NA by arbitrary value
-    isna <- is.na(colval)
-    if(any(isna)) {
-      stuff <- attr(colmap, "stuff")
-      colvalues <- stuff$outputs
-      colval[isna] <- colvalues[1]
-    }
+    ## ## replace NA by arbitrary value
+    ##  isna <- is.na(colval)
+    ## if(any(isna)) {
+    ##   stuff <- attr(colmap, "stuff")
+    ##   colvalues <- stuff$outputs
+    ##   colval[isna] <- colvalues[1]
+    ## }
     ## pass colour matrix (and suppress lines)
     colinfo <- list(col=colval, border=NA)
   } else {
@@ -83,24 +113,17 @@ persp.im <- function(x, ...,
     colval <- t(as.matrix(colval))
     ## strip one row and column for input to persp.default
     colval <- colval[-1, -1]
-    colval[is.na(colval)] <- colvalues[1]
+    ## colval[is.na(colval)] <- colvalues[1]
     ## pass colour matrix (and suppress lines)
     colinfo <- list(col=colval, border=NA)
   }
 
-  if(apron) {
-    ## add an 'apron'
-    zlim <- list(...)$zlim
-    bottom <- if(!is.null(zlim)) zlim[1] else min(x)
-    x <- na.handle.im(x, na.replace=bottom)
-    x <- padimage(x, bottom)
-    xinfo <- summary(x)
-    if(is.matrix(colval <- colinfo$col)) {
-      colval <- matrix(col2hex(colval), nrow(colval), ncol(colval))
+  if(apron && is.matrix(colval <- colinfo$col) && !is.null(isapron)) {
+      ## assign grey colour to apron pixels
       grijs <- col2hex("lightgrey")
-      colval <- cbind(grijs, rbind(grijs, colval, grijs), grijs)
+      isap <- t(as.matrix(isapron))[-1,-1]
+      colval[isap] <- grijs
       colinfo$col <- colval
-    }
   }
 
   if(spatstat.options("monochrome"))
@@ -123,6 +146,10 @@ persp.im <- function(x, ...,
   dotargs <- list(...)
   if(spatstat.options("monochrome"))
     dotargs <- col.args.to.grey(dotargs)
+
+  ## catch argument 'adj.main' and convert to recognised argument 'adj'
+  if(!is.na(k <- match("adj.main", names(dotargs))))
+    names(dotargs)[k] <- "adj"
     
   xcol <- x$xcol
   yrow <- x$yrow
@@ -237,21 +264,23 @@ perspVisible <- function(x, y, z, M) {
   return(V)
 }
 
-perspPoints <- function(x, y=NULL, ..., Z, M) {
+perspPoints <- function(x, y=NULL, ..., Z, M, occluded=TRUE) {
   xy <- xy.coords(x, y)
   stopifnot(is.im(Z))
   X <- as.ppp(xy, W=Frame(Z))
   if(!(is.matrix(M) && all(dim(M) == 4)))
     stop("M should be a 4 x 4 matrix, returned from persp()")
-  V <- attr(M, "visible")
-  if(is.null(V)) {
-    warning(paste("M does not contain visibility information;",
-               "it should be recomputed by persp() with visible=TRUE"))
-  } else {
-    ## restrict to visible points
-    VX <- V[X, drop=FALSE]
-    VX[is.na(VX)] <- FALSE
-    X <- X[VX]
+  if(occluded) {
+    V <- attr(M, "visible")
+    if(is.null(V)) {
+      warning(paste("M does not contain visibility information;",
+                    "it should be recomputed by persp() with visible=TRUE"))
+    } else {
+      ## restrict to visible points
+      VX <- V[X, drop=FALSE]
+      VX[is.na(VX)] <- FALSE
+      X <- X[VX]
+    }
   }
   #' determine heights
   ZX <- Z[X, drop=FALSE] # may contain NA
@@ -260,14 +289,17 @@ perspPoints <- function(x, y=NULL, ..., Z, M) {
 }
 
 perspSegments <- local({
-  perspSegments <- function(x0, y0=NULL, x1=NULL, y1=NULL, ..., Z, M) {
+  perspSegments <- function(x0, y0=NULL, x1=NULL, y1=NULL, ...,
+                            Z, M, occluded=TRUE) {
     stopifnot(is.im(Z))
     if(!(is.matrix(M) && all(dim(M) == 4)))
       stop("M should be a 4 x 4 matrix, returned from persp()")
-    V <- attr(M, "visible")
-    if(is.null(V))
-      warning(paste("M does not contain visibility information;",
-                 "it should be recomputed by persp() with visible=TRUE"))
+    if(occluded) {
+      V <- attr(M, "visible")
+      if(is.null(V))
+        warning(paste("M does not contain visibility information;",
+                      "it should be recomputed by persp() with visible=TRUE"))
+    }
     
     if(is.psp(X <- x0) && is.null(y0) && is.null(x1) && is.null(y1)) {
       eX <- X$ends
@@ -280,7 +312,8 @@ perspSegments <- local({
       check.nvector(y1, naok=TRUE, vname="y1")
       eX <- cbind(x0, y0, x1, y1)
     }
-    if(is.null(V)) {
+    if(!occluded || is.null(V)) {
+      ## no segments will be occluded
       Y <- eX
     } else {
       ## chop each segment to length of single pixel along either axis
@@ -317,21 +350,22 @@ perspSegments <- local({
   perspSegments
 })
 
-perspLines <- function(x, y=NULL, ..., Z, M) {
+perspLines <- function(x, y=NULL, ..., Z, M, occluded=TRUE) {
   xy <- xy.coords(x, y)
   n <- length(xy$x)
-  perspSegments(x[-n], y[-n], x[-1], y[-1], Z=Z, M=M, ...)
+  perspSegments(x[-n], y[-n], x[-1], y[-1], Z=Z, M=M, ..., occluded=occluded)
 }
 
 perspContour <- function(Z, M, ...,
-                         nlevels=10, levels=pretty(range(Z), nlevels)) {
+                         nlevels=10, levels=pretty(range(Z), nlevels),
+                         occluded=TRUE) {
   cl <- contourLines(x=Z$xcol,
                      y=Z$yrow,
                      z=t(Z$v),
                      nlevels=nlevels, levels=levels)
   for(i in seq_along(cl)) {
     cli <- cl[[i]]
-    perspLines(cli$x, cli$y, ..., Z=Z, M=M)
+    perspLines(cli$x, cli$y, ..., Z=Z, M=M, occluded=occluded)
   }
   invisible(NULL)
 }
