@@ -1,7 +1,7 @@
 #
 #  psp.R
 #
-#  $Revision: 1.112 $ $Date: 2024/02/04 08:04:51 $
+#  $Revision: 1.120 $ $Date: 2025/06/17 07:48:49 $
 #
 # Class "psp" of planar line segment patterns
 #
@@ -577,45 +577,63 @@ rotate.psp <- function(X, angle=pi/2, ..., centre=NULL) {
 is.empty.psp <- function(x) { return(x$n == 0) } 
 
 identify.psp <- function(x, ..., labels=seq_len(nsegments(x)),
-                         n=nsegments(x), plot=TRUE) {
+                         n=nsegments(x), plot=TRUE,
+                         paint=plot, paint.args=list()) {
+  if(dev.cur() == 1 && interactive()) {
+    eval(substitute(plot(X), list(X=substitute(x))))
+  }
   Y <- x
-  W <- as.owin(Y)
-  mids <- midpoints.psp(Y)
-  poz <- c(1, 2,4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
+  B <- Frame(Y)
+  Bplus <- grow.rectangle(B, max(sidelengths(B))/4)
+  ## mids <- midpoints.psp(Y)
+  ## poz <- c(1, 2, 4, 3)[(floor(angles.psp(Y)/(pi/4)) %% 4) + 1L]
+  gp <- if(plot) graphicsPars("lines") else NULL
   if(!(is.numeric(n) && (length(n) == 1) && (n %% 1 == 0) && (n >= 0)))
     stop("n should be a single integer")
   out <- integer(0)
   while(length(out) < n) {
-    xy <- spatstatLocator(1)
-    # check for interrupt exit
+    xy <- spatstatLocator(1, type="n")
+    ## check for interrupt exit
     if(length(xy$x) == 0)
       return(out)
-    # find nearest segment
-    X <- ppp(xy$x, xy$y, window=W)
+    ## find nearest segment
+    X <- ppp(xy$x, xy$y, window=Bplus)
     ident <- project2segment(X, Y)$mapXY
-    # add to list
-    if(ident %in% out) {
+    if(length(ident) == 0) {
+      cat("Query location is too far away\n")
+    } else if(ident %in% out) {
       cat(paste("Segment", ident, "already selected\n"))
     } else {
-      if(plot) {
-        # Display
-        mi <- mids[ident]
+      ## add to list
+      if(plot || paint) {
+        ## Display
         li <- labels[ident]
-        po <- poz[ident]
-        mix <- mi$x
-        miy <- mi$y
+        ## mi <- mids[ident]
+        Dother <- distmap(Y[-ident])
+        bestplace <- where.max(Dother * pixellate(Y[ident], what="indicator"))
+        ## po <- poz[ident]
+        mix <- bestplace$x
+        miy <- bestplace$y
         dont.complain.about(li, mix, miy)
-        do.call.matched(graphics::text.default,
-                        resolve.defaults(list(x=quote(mix), 
-					      y=quote(miy), 
-				              labels=quote(li)),
-                                         list(...),
-                                         list(pos=po)))
+        if(plot)
+          do.call.matched(graphics::text.default,
+                          resolve.defaults(list(x=quote(mix), 
+                                                y=quote(miy), 
+                                                labels=quote(li)),
+                                           list(...),
+                                           list(pos=3)))
+        if(paint) 
+          do.call.matched(plot.psp,
+                          resolve.defaults(list(x=Y[ident], add=TRUE),
+                                           paint.args,
+                                           list(col="blue", lwd=2),
+                                           list(...)),
+                          extrargs=gp)
       }
       out <- c(out, ident)
     }
   }
-  # exit if max n reached
+  ## exit if max n reached
   return(out)
 }
 
